/* -*- compile-command: "msdev cylinder.dsp /make \"Cylinder - Win32 Debug\"" -*-
 */

#include <modeler.h>

MODEL(Cylinder);

#define VIEW_X  1
#define VIEW_Y  2
#define VIEW_Z  3

#define XROT_BEFORE  10
#define XROT_AFTER   11
#define TRANSLATE    12
#define SHOWLOCAL    13

int Model_SetupCmd( ClientData clientdata, Tcl_Interp* interp,
		    int objc, Tcl_Obj* const objv[] )
{
    scale( interp, VIEW_X, "x view", 0.0, 360.0, 1.0, 296.0 );
    scale( interp, VIEW_Y, "y view", 0.0, 360.0, 1.0, 0.0 );
    scale( interp, VIEW_Z, "z view", 0.0, 360.0, 1.0, 241.0 );
    
    scale( interp, XROT_BEFORE, "x rotation before", 0.0, 360.0, 1.0, 0.0 );
    checkbox( interp, TRANSLATE, "translate", 0 );
    scale( interp, XROT_AFTER, "x rotation after", 0.0, 360.0, 1.0, 0.0 );

    checkbox( interp, SHOWLOCAL, "show local coordinates", 0 );
    
    return TCL_OK;
}

int Model_ResizeCmd( ClientData clientdata, Tcl_Interp* interp,
		     int objc, Tcl_Obj* const objv[] )
{
    int width, height;

    if ( objc != 3 )
    {
	Tcl_SetResult( interp, "wrong # args: should be \"width height\"",
		       TCL_STATIC );
	return TCL_ERROR;
    }

    if ( Tcl_GetIntFromObj( interp, objv[1], &width ) != TCL_OK )
	return TCL_ERROR;
    if ( Tcl_GetIntFromObj( interp, objv[2], &height ) != TCL_OK )
	return TCL_ERROR;

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    gluPerspective( 30.0, (double)width/height, 1.0, 100.0 );
    glViewport( 0, 0, width, height );

    glClearColor( 0.0, 0.0, 0.0, 0.0 );

    return TCL_OK;
}

int Model_RedrawCmd( ClientData clientdata, Tcl_Interp* interp,
		     int objc, Tcl_Obj* const objv[] )
{
    int lighting;
    
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    draw_mode_setup();

    glTranslated( 0.0, 0.0, -15.0 );
    glRotated( get_control_d( VIEW_X ), 1.0, 0.0, 0.0 );
    glRotated( get_control_d( VIEW_Y ), 0.0, 1.0, 0.0 );
    glRotated( get_control_d( VIEW_Z ), 0.0, 0.0, 1.0 );

    glGetIntegerv( GL_LIGHTING, &lighting );
    glDisable( GL_LIGHTING );

    glLineWidth( 5.0 );
    
    if ( get_control_b( SHOWLOCAL ) )
    {
	glBegin( GL_LINES );
	glColor3d( 1.0, 0.7, 0.7 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 5.0, 0.0, 0.0 );
	glColor3d( 0.7, 1.0, 0.7 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 0.0, 5.0, 0.0 );
	glColor3d( 0.7, 0.7, 1.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 0.0, 0.0, 5.0 );
	glEnd();

	glPushMatrix();
	glRotated( get_control_d( XROT_BEFORE ), 1.0, 0.0, 0.0 );
	glTranslated( 0.01, 0.01, 0.01 );
	if ( get_control_b( TRANSLATE ) )
	    glTranslated( 3.0, 1.0, 0.0 );
	glRotated( get_control_d( XROT_AFTER ), 1.0, 0.0, 0.0 );

	glBegin( GL_LINES );
	glColor3d( 1.0, 0.0, 0.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 2.0, 0.0, 0.0 );
	glColor3d( 0.0, 1.0, 0.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 0.0, 2.0, 0.0 );
	glColor3d( 0.0, 0.0, 1.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 0.0, 0.0, 2.0 );
	glEnd();

	glPopMatrix();
    }
    else
    {
	glBegin( GL_LINES );
	glColor3d( 1.0, 0.0, 0.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 5.0, 0.0, 0.0 );
	glColor3d( 0.0, 1.0, 0.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 0.0, 5.0, 0.0 );
	glColor3d( 0.0, 0.0, 1.0 );
	glVertex3d( 0.0, 0.0, 0.0 );
	glVertex3d( 0.0, 0.0, 5.0 );
	glEnd();
    }

    
    if ( lighting == GL_TRUE )
	glEnable( GL_LIGHTING );

    diffuse_color( 0.5, 0.5, 0.5 );

    glRotated( get_control_d( XROT_BEFORE ), 1.0, 0.0, 0.0 );
    if ( get_control_b( TRANSLATE ) )
	glTranslated( 3.0, 1.0, 0.0 );
    glRotated( get_control_d( XROT_AFTER ), 1.0, 0.0, 0.0 );

    glTranslated( 0.0, 0.0, -1.0 );
    cylinder( 2.0, 0.5, 0.5 );

    return TCL_OK;
}


